unit db_users;

{
Программа "Видеочат"
Copyright (c) 2020 Любезный В. В.
https://www.lubezniy.ru/soft/videochat

Модуль с описанием сущностей и класса базы данных пользователей.
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type

  { тип записи пользователя в БД }
  TUserDBRec = record
    UID: string; // уникальный идентификатор
    Login: string; // логин
    Password: string; // пароль
    displayName: string; // отображаемое имя
    lastLogin: int64; // последний логин
  end;

  // тип объекта БД пользователей
  TUserDB = object
    Recs: array of TUserDBRec;
    procedure Read;
    // чтение файла конфигурации пользователей
    function Authenticate(Login, Password: string): integer;
    // аутентификация пользователя; при ошибке возвращается -1
    function IndexByUID(UID: string): integer;
    // индекс пользователя в записях по UID (-1, если не нашли)
    function onlineCount: integer; // кол-во пользователей онлайн
  end;

implementation

uses fpjson, jsonparser, jsonscanner, blcksock;

procedure TUserDB.Read;
var
  f: TextFile;
  s, s1: string;
  JSONParser: TJSONParser;
  Data: TJSONData;
  i: integer;
begin
  // читаем json-файл
  AssignFile(f, ExtractFilePath(ParamStr(0)) + 'config/users.json');
  reset(f);
  s := '';
  while not EOF(f) do
  begin
    readln(f, s1);
    if (s1 <> '') then
    begin
      // Добавляем нестандартную возможность комментирования в JSON-файле. Комментарий начинается с двух символов дроби.
      if (Copy(s1, 1, 2) <> '//') then
        s := s + s1 + CRLF;
    end;
  end;
  CloseFile(f);
  JSONParser := TJSONParser.Create(s, DefaultOptions);
  Data := JSONParser.Parse;
  JSONParser.Free;
  SetLength(Recs, 0);
  // распарсили файл - заполняем структуры данных
  if Data.FindPath('Users').Count > 0 then
    for i := 0 to Data.FindPath('Users').Count - 1 do
    begin
      SetLength(Recs, i + 1);
      Recs[i].UID := Data.FindPath('Users[' + IntToStr(i) + '].UID').AsString;
      Recs[i].Login := Data.FindPath('Users[' + IntToStr(i) + '].Login').AsString;
      Recs[i].Password := Data.FindPath('Users[' + IntToStr(i) + '].Password').AsString;
      Recs[i].DisplayName := Data.FindPath('Users[' + IntToStr(i) +
        '].DisplayName').AsString;
      Recs[i].LastLogin := 0;
    end;
  // освобождаем память
  Data.Free;
end;

function TUserDB.Authenticate(Login, Password: string): integer;
var
  i: integer; // счётчик записей для перебора
begin
  Result := -1;
  if (Login <> '') and (Password <> '') and (Length(Recs) > 0) then
    for i := 0 to Length(Recs) - 1 do
      if Recs[i].Login = Login then
      begin
        if Recs[i].Password = Password then
        begin
          Result := i;
          Recs[i].lastLogin := Trunc((Now - EncodeDate(1970, 1, 1)) * 24 * 60 * 60);
        end;
        Break;
      end;
end;

function TUserDB.IndexByUID(UID: string): integer;
var
  i: integer; // счётчик записей для перебора
begin
  Result := -1;
  if Length(Recs) > 0 then
    for i := 0 to Length(Recs) - 1 do
      if Recs[i].UID = UID then
      begin
        Result := i;
        Break;
      end;
end;

function TUserDB.onlineCount: integer;
var
  i: integer; // счётчик записей для перебора
  ts: int64; // UNIX timestamp текущего времени
begin
  ts := Trunc((Now - EncodeDate(1970, 1, 1)) * 24 * 60 * 60);
  Result := 0;
  if Length(Recs) > 0 then
    for i := Length(Recs) - 1 downto 0 do
      if Recs[i].lastLogin >= ts - 10 then
        Inc(Result);
end;

end.
