unit postdata_class;

{
Программа "Видеочат"
Copyright (c) 2020 Любезный В. В.
https://www.lubezniy.ru/soft/videochat

Модуль описания сущностей и класса работы с данными POST-запроса
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpjson, jsonparser;

type

  { TPOSTDataRec }
  TPOSTDataRec = record
    Name: string;
    Value: string;
  end;

  { TPOSTData - данные POST }
  TPOSTData = class
    Recs: array of TPOSTDataRec;
    JSONData: TJSONData;
  private
    procedure Init;
    function dataToString(Data: TMemoryStream): string;
    function readln(var s: string): string;
  public
    function Read(Data: TMemoryStream; ContentType: string): boolean;
  end;

implementation

uses stringfuncs, jsonscanner;

procedure TPOSTData.Init;
begin
  SetLength(Recs, 0);
end;

function TPOSTData.dataToString(Data: TMemoryStream): string;
var
  Buf: array[1..1024] of ANSIChar;
  Count, i: integer;
begin
  Result := '';
  repeat
    Count := Data.Read(Buf, 1024);
    if (Count > 0) then
      for i := 1 to Count do
        Result := Result + Buf[i];
  until Count < 1024;
end;

function TPOSTData.readln(var s: string): string;
begin
  Result := '';
  while ((Length(s) > 0) and (Copy(s, 1, 1) <> Chr(13))) do
  begin
    Result := Result + Copy(s, 1, 1);
    s := Copy(s, 2, Length(s) - 1);
  end;
  if (Length(s) > 0) then
  begin
    if (Copy(s, 1, 1) = Chr(13)) then
      s := Copy(s, 2, Length(s) - 1);
    if (Copy(s, 1, 1) = Chr(10)) then
      s := Copy(s, 2, Length(s) - 1);
  end;
end;

function TPOSTData.Read(Data: TMemoryStream; ContentType: string): boolean;
var
  i, p: integer;
  s, t: string;
  Rec: TPOSTDataRec;
  Boundary: string;
  HeaderName, HeaderValue, AddData: string;
  JSONParser: TJSONParser;
begin
  JSONData := nil;
  if (UpperCase(Copy(ContentType, 1, 16)) = UpperCase('application/json')) then
  begin
    JSONParser := TJSONParser.Create(Data, DefaultOptions);
    JSONData := JSONParser.Parse;
    FreeAndNil(JSONParser);
  end
  else if (UpperCase(Copy(ContentType, 1, 19)) = UpperCase('multipart/form-data')) then
  begin
    // вычисляем boundary
    i := Pos('boundary=', ContentType);
    if i > 0 then
    begin
      i := i + 9;
      Boundary := Copy(ContentType, i, Length(ContentType) - i + 1);
      // парсим данные
      s := dataToString(Data);
      // сначала ищем и обрезаем первый boundary
      // ищем первый Boundary
      p := Pos(Boundary, s);
      // обрезаем его
      i := p + Length(Boundary);
      s := Copy(s, i, Length(s) - i + 1);
      // обрезаем последующие chr(13) и chr(10)
      while ((s[1] = Chr(13)) or (s[1] = Chr(10))) do
        s := Copy(s, 2, Length(s) - 1);
      // основной цикл чтения данных
      while (s <> '') do
      begin
        Rec.Name := '';
        Rec.Value := '';
        // цикл чтения заголовков
        repeat
          // читаем строку заголовка
          t := readln(s);
          if t <> '' then
          begin
            // бьём заголовок на имя и значение
            p := Pos(': ', t);
            HeaderName := Copy(t, 1, p - 1);
            HeaderValue := Copy(t, p + 2, Length(t) - p - 1);
            // бьём значение по точке с запятой, если есть
            p := Pos(';', HeaderValue);
            if p = 0 then
              AddData := ''
            else
            begin
              AddData := Copy(HeaderValue, p + 1, Length(HeaderValue) - p);
              HeaderValue := Copy(HeaderValue, 1, p - 1);
            end;
            if HeaderName = 'Content-Disposition' then
            begin
              if HeaderValue = 'form-data' then
              begin
                // тащим из AddData имя поля, представленное в виде name="имя"
                // обрезаем всё, что до имени
                p := Pos('name="', AddData);
                AddData := Copy(AddData, p + 6, Length(AddData) - 5);
                // ищем закрывающую кавычку
                p := Pos('"', AddData);
                // заполняем сообщение
                Rec.Name := Copy(AddData, 1, p - 1);
              end;
            end;
          end;
          // пустая строка - конец заголовков, начало данных
        until t = '';
        if s <> '' then
        begin
          // ищем позицию Boundary от начала строки
          p := Pos(Boundary, s);
          // ищем позицию конца данных
          while Copy(s, p, 1) <> Chr(13) do
            Dec(p);
          Dec(p);
          // заполняем данные
          Rec.Value := Copy(s, 1, p);
          // обрезаем строку по концу Boundary
          Inc(p, 3);
          Inc(p, Length(Boundary));
          s := Copy(s, p + 2, Length(s) - p - 1);
          // если есть запись, добавляем её
          if Rec.Name <> '' then
          begin
            i := Length(Recs);
            SetLength(Recs, i + 1);
            Recs[i] := Rec;
          end;
          // тримим s слева
          while ((Copy(s, 1, 1) = Chr(13)) or (Copy(s, 1, 1) = Chr(10))) do
            s := Copy(s, 2, Length(s) - 1);
        end;
      end;
    end;
  end
  else if (UpperCase(Copy(ContentType, 1, 33)) =
    UpperCase('application/x-www-form-urlencoded')) then
  begin
    s := dataToString(Data);
    while (s <> '') do
    begin
      i := Pos('&', s);
      if i > 0 then
      begin
        t := Copy(s, 1, i - 1);
        s := Copy(s, i + 1, Length(s) - i);
      end
      else
      begin
        t := s;
        s := '';
      end;
      // теперь бьём t по знаку = на имя и значение
      i := Pos('=', t);
      if i > 0 then
      begin
        Rec.Name := Copy(t, 1, i - 1);
        Rec.Value := URLDecode(Copy(t, i + 1, Length(t) - i));
        i := Length(Recs);
        SetLength(Recs, i + 1);
        Recs[i] := Rec;
      end;
    end;
    Result := True;
  end;


 {
 Content-Type: multipart/form-data; boundary=424068705621616454482848167198
 Data:
 -----------------------------424068705621616454482848167198
 Content-Disposition: form-data; name="data1"

 Тест
 -----------------------------424068705621616454482848167198
 Content-Disposition: form-data; name="data2"

 Тест2
 -----------------------------424068705621616454482848167198--
 }

end;

end.
