unit requestheaders_class;

{
Программа "Видеочат"
Copyright (c) 2020 Любезный В. В.
https://www.lubezniy.ru/soft/videochat

Описание сущностей и класса для парсинга заголовков HTTP-запроса и кук
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type
  TRequestHeaderRec = record
    Name: string;
    Value: string;
  end;

  TRequestHeaders = class
    Recs, Cookies: array of TRequestHeaderRec;
  private
    procedure Init;
    procedure ParseCookies(Value: string);
  public
    function Read(Headers: TStringList): boolean;
    function getValue(Name: string; var Value: string): boolean;
    function getCookieValue(Name: string; var Value: string): boolean;
  end;

implementation

procedure TRequestHeaders.Init;
begin
  SetLength(Recs, 0);
  SetLength(Cookies, 0);
end;

procedure TRequestHeaders.ParseCookies(Value: string);
var
  i, l: integer;
  s, t: string;
  Rec: TRequestHeaderRec;
begin
  // _ym_uid=1523045679169333905; l…b522a1; _ym_visorc_11340277=w
  s := Value;
  while (s <> '') do
  begin
    i := Pos('; ', s);
    if i > 0 then
    begin
      // есть ещё кука - ставим её в t и отрезаем от s
      t := Copy(s, 1, i - 1);
      s := Copy(s, i + 2, Length(s) - i - 1);
    end
    else
    begin
      // кука последняя - просто берём её в t и опустошаем s
      t := s;
      s := '';
    end;
    // парсим полученную куку
    i := Pos('=', t);
    if i > 0 then
    begin
      // добавляем куку в массив
      Rec.Name := Copy(t, 1, i - 1);
      Rec.Value := Copy(t, i + 1, Length(t) - i);
      l := Length(Cookies);
      SetLength(Cookies, l + 1);
      Cookies[l] := Rec;
    end;
  end;
end;

function TRequestHeaders.Read(Headers: TStringList): boolean;
var
  i, l, p: integer;
begin
  Init;
  Result := True;
  if Headers.Count > 0 then
  begin
    for i := 0 to Headers.Count - 1 do
    begin
      p := Pos(': ', Headers.Strings[i]);
      if (p = 0) then
      begin
        Result := False;
        Break;
      end;
      l := Length(Recs);
      SetLength(Recs, l + 1);
      Recs[l].Name := Copy(Headers.Strings[i], 1, p - 1);
      Recs[l].Value := Copy(Headers.Strings[i], p + 2, Length(Headers.Strings[i]) - p - 1);
      // проверяем, чей это заголовок
      if (UpperCase(Recs[l].Name) = 'COOKIE') then
        ParseCookies(Recs[i].Value);
    end;
  end;
end;

function TRequestHeaders.getValue(Name: string; var Value: string): boolean;
var
  i: integer;
  s: string;
begin
  Result := False;
  s := Uppercase(Name);
  if Length(Recs) > 0 then
    for i := 0 to Length(Recs) - 1 do
      if s = UpperCase(Recs[i].Name) then
      begin
        Value := Recs[i].Value;
        Result := True;
        Break;
      end;
end;

function TRequestHeaders.getCookieValue(Name: string; var Value: string): boolean;
var
  i: integer;
begin
  Result := False;
  if Length(Cookies) > 0 then
    for i := 0 to Length(Cookies) - 1 do
      if Name = Cookies[i].Name then
      begin
        Value := Cookies[i].Value;
        Result := True;
        Break;
      end;
end;

end.
