unit stringfuncs;

{
Программа "Видеочат"
Copyright (c) 2020 Любезный В. В.
https://www.lubezniy.ru/soft/videochat

Модуль с вспомогательными функциями для работы со строками
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

// декодирование строки, закодированной в URL (копипастинг из https://forum.lazarus.freepascal.org/index.php?topic=9184.0)
function URLDecode(const s: string): string;

// преобразование даты-времени для куки (копипастинг из https://wiki.freepascal.org/RFC_1123_Time_Format)
function RFC1123TimeFormat(aDateTime: TDateTime; isLocalTime: boolean): string;

// функция вычисления позиции подстроки в строке
function strpos(substr, str: string; startpos: longint): longint;

// функция триммирования строки
function trim(const Line: string): string;

// функция замены подстроки на другую с подсчётом количества
function str_replace(s, ToFind, ToReplace: string; var Counter: integer): string;

// функция генерации (условно) уникального идентификатора
function generateID: string;

implementation

uses DateUtils;

// Datetime to RFC1123 format - from  https://wiki.freepascal.org/RFC_1123_Time_Format
function RFC1123TimeFormat(aDateTime: TDateTime; isLocalTime: boolean): string;
const
  ShortDayNamesEnglish: array[1..7] of string =
    ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
  ShortMonthNamesEnglish: array[1..12] of string = ('Jan', 'Feb', 'Mar', 'Apr',
    'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
var
  day, month: string;
  someDateTime: TDateTime;
begin
  if isLocalTime then
    someDateTime := LocalTimeToUniversal(aDateTime)
  else
    someDateTime := aDateTime;
  day := ShortDayNamesEnglish[DayOfWeek(someDateTime)];
  month := ShortMonthNamesEnglish[MonthOf(someDateTime)];
  Result := day + ', ' + FormatDateTime('dd', someDateTime) + ' ' +
    month + ' ' + FormatDateTime('yyyy hh:nn:ss', someDateTime) + ' GMT';
end;

// Convert URLEncoded string to utf8 string - from https://forum.lazarus.freepascal.org/index.php?topic=9184.0
function URLDecode(const s: string): string;
var
  sAnsi: string;
  sUtf8: string;
  sWide: WideString;

  i, len: cardinal;
  ESC: string[2];
  CharCode: integer;
  c: char;
begin
  sAnsi := PChar(s);
  SetLength(sUtf8, Length(sAnsi));
  i := 1;
  len := 1;
  while (i <= cardinal(Length(sAnsi))) do
  begin
    if (sAnsi[i] <> '%') then
    begin
      if (sAnsi[i] = '+') then
      begin
        c := ' ';
      end
      else
      begin
        c := sAnsi[i];
      end;
      sUtf8[len] := c;
      Inc(len);
    end
    else
    begin
      Inc(i);
      ESC := Copy(sAnsi, i, 2);
      Inc(i, 1);
      try
        CharCode := StrToInt('$' + ESC);
        c := char(CharCode);
        sUtf8[len] := c;
        Inc(len);
      except
      end;
    end;
    Inc(i);
  end;
  Dec(len);
  SetLength(sUtf8, len);

  sWide := UTF8Decode(sUtf8);
  len := Length(sWide);

  Result := sWide;
end;

// функция триммирования строки
function trim(const Line: string): string;
var
  r: string;
  b: boolean;
begin
  r := Line;
  if (r <> '') then
  begin
    // с начала
    repeat
      b := False;
      if r <> '' then
        if (r[1] = #13) or (r[1] = #10) or (r[1] = #32) or (r[1] = #9) then
          b := True;
      if b then
        r := copy(r, 2, Length(r) - 1);
    until b = False;
    // с конца
    repeat
      b := False;
      if r <> '' then
        if (r[Length(r)] = #13) or (r[Length(r)] = #10) or (r[Length(r)] = #32) or
          (r[Length(r)] = #9) then
          b := True;
      if b then
        r := Copy(r, 1, Length(r) - 1);
    until b = False;
  end;
  Result := r;
end;

// функция вычисления позиции подстроки в строке
function strpos(substr, str: string; startpos: longint): longint;
var
  l, m: longint;
begin
  l := 0;
  if startpos > Length(str) - Length(substr) + 1 then
    Result := 0
  else
  begin
    for m := startpos to Length(str) - Length(substr) + 1 do
      if (Copy(str, m, Length(substr)) = substr) then
      begin
        l := m;
        Break;
      end;
    Result := l;
  end;
end;

// функция замены подстроки на другую с подсчётом количества
function str_replace(s, ToFind, ToReplace: string; var Counter: integer): string;
var
  a: string;
  i, j: integer;
begin
  Counter := 0;
  if Pos(ToFind, s) = 0 then
    Result := s
  else
  begin
    a := s;
    i := 1;
    repeat
      j := strpos(ToFind, a, i);
      if j > 0 then
      begin
        Inc(Counter);
        i := j + Length(ToReplace);
        a := Copy(a, 1, j - 1) + ToReplace +
          Copy(a, j + Length(ToFind), Length(a) - j - Length(ToFind) + 1);
      end;
    until j = 0;
    Result := a;
  end;
end;

// функция генерации случайного идентификатора
function generateID: string;
const
  hexCharArr: array[0..15] of char =
    ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f');
var
  i: integer;
begin
  Result := '';
  Randomize;
  for i := 1 to 32 do
    Result := Result + hexCharArr[Random(16)];
end;

end.
