unit webserverconfig_class;

{
Программа "Видеочат"
Copyright (c) 2020 Любезный В. В.
https://www.lubezniy.ru/soft/videochat

Модуль с описанием сущностей и класса конфигурации web-сервера
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type

  { тип записи статического файла }
  TStaticFileURI = record
    URI: string; // URI, по которому нужно отдать файл
    FilePath: string; // путь к файлу относительно exe
    ContentType: string; // тип контента
    RequireAuthorized: boolean;
    // отдавать файл только авторизованным пользователям
  end;

  { собственно класс конфигурации }
  TWebServerConfig = object
    DebugMode: boolean; // отладочный режим
    LingerTimeout: integer;
    // таймаут ожидания соединения в миллисекундах
    LingerEnable: boolean; // включить Linger
    BindAddress: string;
    // IP-адрес для связывания с серверным сокетом
    BindPort: integer;
    // порт для связывания с серверным сокетом
    CanReadTimeout: integer;
    // таймаут ожидания данных в миллисекундах
    ReceiveTimeout: integer;
    // таймаут приёма данных в миллисекундах
    OpenSSLConfigFile: string; // путь к файлу openssl.cfg
    CACertificateFile: string;
    // полное имя файла сертификата УЦ, выдавшего SSL-сертификат
    CertificateFile: string;
    // полное имя файла нашего SSL-сертификата
    PrivateKeyFile: string;
    // полное имя файла закрытого ключа
    PrivateKeyPassword: string; // пароль к закрытому ключу
    VerifySSL: boolean;
    // флаг проверки сертификата абонента
    MaxRequestBodySize: longint;
    // макс. размер тела запроса в байтах
    StaticFileURIs: array of TStaticFileURI; // URI статических файлов
    procedure Read; // прочитать конфиг
    function getStaticFileIndex(URI: string; isUserAuthorized: boolean): integer;
    // отыскать запись статического файла по URI
  end;

implementation

uses fpjson, jsonparser, jsonscanner, blcksock;

procedure TWebServerConfig.Read;
var
  f: TextFile;
  s, s1: string;
  JSONParser: TJSONParser;
  Data, JD: TJSONData;
  i: integer;
begin
  // Очищаем массив статических файлов
  SetLength(StaticFileURIs, 0);
  // Читаем json-файл в строку s.
  AssignFile(f, ExtractFilePath(ParamStr(0)) + 'config/webserver.json');
  reset(f);
  s := '';
  while not EOF(f) do
  begin
    readln(f, s1);
    if (s1 <> '') then
    begin
      // Добавляем нестандартную возможность комментирования в JSON-файле. Комментарий начинается с двух символов дроби.
      if (Copy(s1, 1, 2) <> '//') then
        s := s + s1 + CRLF;
    end;
  end;
  CloseFile(f);
  // Парсим прочитанный контент.
  JSONParser := TJSONParser.Create(s, DefaultOptions);
  Data := JSONParser.Parse;
  JSONParser.Free;
  // Заполняем свойства объектов данными из файла - с учётом того, что каких-то параметров может не быть.
  JD := Data.FindPath('DebugMode');
  if JD = nil then
  begin
    DebugMode := False;
  end
  else
  begin
    DebugMode := JD.AsBoolean;
  end;
  JD := Data.FindPath('LingerEnable');
  if JD = nil then
  begin
    LingerEnable := True;
  end
  else
  begin
    LingerEnable := JD.AsBoolean;
  end;
  JD := Data.FindPath('LingerTimeout');
  if JD = nil then
  begin
    LingerTimeout := 10000;
  end
  else
  begin
    LingerTimeout := JD.AsInteger;
  end;
  JD := Data.FindPath('BindAddress');
  if JD = nil then
  begin
    BindAddress := '0.0.0.0';
  end
  else
  begin
    BindAddress := JD.AsString;
  end;
  JD := Data.FindPath('BindPort');
  if JD = nil then
  begin
    BindPort := 443;
  end
  else
  begin
    BindPort := JD.AsInteger;
  end;
  JD := Data.FindPath('CanReadTimeout');
  if JD = nil then
  begin
    CanReadTimeout := 1000;
  end
  else
  begin
    CanReadTimeout := JD.AsInteger;
  end;
  JD := Data.FindPath('ReceiveTimeout');
  if JD = nil then
  begin
    ReceiveTimeout := 120000;
  end
  else
  begin
    ReceiveTimeout := JD.AsInteger;
  end;
  JD := Data.FindPath('OpenSSLConfigFile');
  if JD = nil then
  begin
    OpenSSLConfigFile := ParamStr(0) + 'openssl.cnf';
  end
  else
  begin
    OpenSSLConfigFile := JD.AsString;
  end;
  JD := Data.FindPath('CACertificateFile');
  if JD = nil then
  begin
    CACertificateFile := '';
  end
  else
  begin
    CACertificateFile := JD.AsString;
  end;
  JD := Data.FindPath('CertificateFile');
  if JD = nil then
  begin
    CertificateFile := '';
  end
  else
  begin
    CertificateFile := JD.AsString;
  end;
  JD := Data.FindPath('PrivateKeyFile');
  if JD = nil then
  begin
    PrivateKeyFile := '';
  end
  else
  begin
    PrivateKeyFile := JD.AsString;
  end;
  JD := Data.FindPath('PrivateKeyPassword');
  if JD = nil then
  begin
    PrivateKeyPassword := '';
  end
  else
  begin
    PrivateKeyPassword := JD.AsString;
  end;
  JD := Data.FindPath('VerifySSL');
  if JD = nil then
  begin
    VerifySSL := False;
  end
  else
  begin
    VerifySSL := JD.AsBoolean;
  end;
  JD := Data.FindPath('MaxRequestBodySize');
  if JD = nil then
  begin
    MaxRequestBodySize := 1048576;
  end
  else
  begin
    MaxRequestBodySize := JD.AsInt64;
  end;
  if Data.FindPath('StaticFileURIs').Count > 0 then
    for i := 0 to Data.FindPath('StaticFileURIs').Count - 1 do
    begin
      SetLength(StaticFileURIs, i + 1);
      StaticFileURIs[i].ContentType :=
        Data.FindPath('StaticFileURIs[' + IntToStr(i) + '].ContentType').AsString;
      StaticFileURIs[i].FilePath :=
        Data.FindPath('StaticFileURIs[' + IntToStr(i) + '].FilePath').AsString;
      StaticFileURIs[i].URI := Data.FindPath('StaticFileURIs[' +
        IntToStr(i) + '].URI').AsString;
      StaticFileURIs[i].RequireAuthorized :=
        Data.FindPath('StaticFileURIs[' + IntToStr(i) + '].RequireAuthorized').AsBoolean;
    end;
  // освобождаем память
  Data.Free;
end;

function TWebServerConfig.getStaticFileIndex(URI: string;
  isUserAuthorized: boolean): integer;
var
  i, j: integer;
begin
  Result := -1;
  j := -1;
  if Length(StaticFileURIs) > 0 then
    for i := 0 to Length(StaticFileURIs) - 1 do
      if StaticFileURIs[i].URI = URI then
      begin
        j := i;
        Break;
      end;
  if j <> -1 then
    if (not StaticFileURIs[j].RequireAuthorized) or isUserAuthorized then
      Result := j;
end;

end.
